class PanelsController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  before_filter :authenticate_user!, :only => [:index, :show, :new, :edit, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]

  private
  
  def treat_param panel
    panel.author_id = @author.id
  end
  
  public
  
  # GET /panels
  # GET /panels.json
  def index
    @page = Panel.page params[:page]
    @page_size = Panel.page_size params[:page_size]
    @panels = Panel.list({}, @page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @panels.to_json(Panel.list_json_opt) }
    end
  end

  # GET /panels/1
  # GET /panels/1.json
  def show
    @panel = Panel.show(params[:id], @author)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @panel.to_json(Panel.show_json_include_opt) }
      format.jsonp {
        render :json => "callback(" + @panel.to_json(Panel.show_json_include_opt) + ");"
      }
    end
  end

  def count
    @panel = {:count => Panel.visible_count}
    respond_to do |format|
      format.json { render json: @panel.to_json }
    end
  end
  
  def list
    @panels = Panel.all :order => 'updated_at'

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @panels }
    end
  end

  def browse
    @panel = Panel.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
    end
  end

  def new
    @panel = Panel.new
    @panel.supply_default @author

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @panel }
    end
  end

  def edit
    @panel = Panel.show(params[:id], @author)
    respond_to do |format|
      format.html
      format.js
    end
  end

  # POST /panels
  # POST /panels.json
  def create
    if params[:json]
      jsn = JSON.parse(params[:json])
    end
    @prm = params[:panel] || jsn
    @panel = Panel.new(@prm)
    treat_param @panel

    respond_to do |format|
      if @panel.store
        format.html { redirect_to @panel, notice: 'Panel was successfully created.' }
        format.json { render json: @panel, status: :created, location: @panel }
      else
        format.html { render action: "new" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /panels/1
  # PUT /panels/1.json
  def update
    @panel = Panel.show(params[:id], @author)
    if params[:json]
      jsn = JSON.parse(params[:json])
    end
    @prm = params[:panel] || jsn
    respond_to do |format|
      @panel.attributes = @prm
      if @panel.store
        format.html { redirect_to @panel, notice: 'Panel was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /panels/1
  # DELETE /panels/1.json
  def destroy
    @panel = Panel.find(params[:id])
    if @panel.own? @author
      respond_to do |format|
        Panel.transaction do
          @panel.destroy_and_shorten
          format.html { redirect_to panels_url }
          format.json { head :ok }
        end
      end
    else
      format.html { render action: "edit" }
      format.json { render json: @panel.errors, status: :unprocessable_entity }
    end
  end
  
  def move
    @panel = Panel.find(params[:id])
    @new_seq = params[:panel][:t].to_i
    respond_to do |format|
      Panel.transaction do
        if @panel.move_to(@new_t)
          format.html { redirect_to @panel, notice: 'Panel was successfully moved.' }
          format.json { head :ok }
        else
          format.html { render action: "show" }
          format.json { render json: @panel.errors, status: :unprocessable_entity }
        end
      end
    end
  end
  
end
