#コマ
class Panel < ActiveRecord::Base
  belongs_to :author
  belongs_to :resource_picture
#  belongs_to :background_picture, :class_name => 'ResourcePicture'
  has_many :stories
  has_many :panel_pictures, :dependent => :destroy
  has_many :speech_balloons, :dependent => :destroy
  accepts_nested_attributes_for :panel_pictures, :allow_destroy => true
  accepts_nested_attributes_for :speech_balloons, :allow_destroy => true

  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :border, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  validates :x, :numericality => {:allow_blank => true}
  validates :y, :numericality => {:allow_blank => true}
  validates :z, :numericality => {:allow_blank => true, :greater_than => 0}
  validates :author_id, :presence => true, :numericality => true, :existence => true
  
  def supply_default au
    return false unless au
    self.border = 0 if self.border.blank?
    self.author_id = au.id
  end
  
  def self.collect_element_value elements, name, ex_nil = false
    e = elements.map {|e|
      e.map {|o|
        o[name]
      }
    }.flatten
    e = e.compact if ex_nil
    e
  end
  
  def self.validate_id ary, pid
    ary.map {|v|
      if pid
        (v == pid or v == nil) ? nil : false
      else
        v ? false : nil
      end
    }.compact.empty?
  end
  
  def self.validate_element_id elements, name, pid
    Panel.validate_id(Panel.collect_element_value(elements, name), pid)
  end
  
  def self.validate_elements_id c
    c.map {|conf|
      Panel.validate_element_id(conf[:elements], conf[:name], conf[:parent_id]) ? nil : false
    }.compact.empty?
  end
  
  def self.validate_t ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_element_t elements, name
    Panel.validate_t(Panel.collect_element_value(elements, name, true))
  end
  
  def self.validate_elements_t c
    c.map {|conf|
      Panel.validate_element_t(conf[:elements], conf[:name]) ? nil : false
    }.compact.empty?
  end
  
  def validate_id_list
    r = self.speech_balloons.map {|sb|
      {:elements => [sb.speeches, sb.balloons], :name => :speech_balloon_id, :parent_id => sb.id}
    }
    r.unshift({:elements => [self.panel_pictures, self.speech_balloons], :name => :panel_id, :parent_id => self.id})
    r
  end
  
  def validate_child
    r1 = Panel.validate_elements_id validate_id_list
    r2 = Panel.validate_elements_t [
      {:elements => [self.panel_pictures, self.speech_balloons], :name => :t}
    ]
    r1 and r2
  end
  
  def store
    res = false
    Panel.transaction do
      unless validate_child
        self.errors.add :base , 'invalid time'
        raise ActiveRecord::Rollback
      end
      res = self.save
    end
    res
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:order => 'panels.updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    Panel.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {
      :panel_pictures => {
        :resource_picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :author => {}
    }}
  end
  
  def self.list_json_opt
    {:include => {
      :panel_pictures => {
        :resource_picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :author => {}
    }}
  end
  
  def self.show rid, au, opt = {}
    r = Panel.find(rid, :include => self.show_include_opt(opt))
    raise ActiveRecord::Forbidden unless r.visible?(au)
    r
  end
  
  def self.show_include_opt opt = {}
    res = {
      :panel_pictures => {
        :resource_picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :author => {}
    }
    res.merge!(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => {
      :panel_pictures => {
        :resource_picture => {:artist => {}, :license => {}}
      }, 
      :speech_balloons => {:balloons => {}, :speeches => {}},
      :author => {}
    }}
  end
  
  def visible? au
    return false unless au
    self.own?(au) or self.publish?
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def usable? au
    own? au
  end
  
  def publish?
    self.publish > 0
  end
  
  def sort_by_time
    pe = []
    self.panel_pictures.each do |picture|
      pe[picture.t] = picture
    end
    self.speech_balloons.each do |sb|
      pe[sb.t] = sb
    end
    pe
  end
  
  def each_element
    self.sort_by_time.each do |e|
      yield e
    end
  end
  
  def panel_elements
    res = []
    self.each_element do |elm|
      if elm.kind_of?(PanelPicture)
        res[elm.t] = elm.to_json({:include => :resource_picture})
      end
      if elm.kind_of?(SpeechBalloon)
        res[elm.t] = elm.to_json({:include => {:balloons => {}, :speeches => {}}})
      end
    end
    res
  end
  
  def to_json_play
    self.to_json :methods => :panel_elements
  end
  
  def self.visible_count
    Panel.count
  end
  
end
