#ストーリー
class Story < ActiveRecord::Base
  belongs_to :author
  belongs_to :panel
  belongs_to :comic
  
  validates :comic_id, :presence => true, :numericality => true, :existence => true
  validates :panel_id, :presence => true, :numericality => true, :existence => true
  validates :author_id, :presence => true, :numericality => true, :existence => true
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def supply_default
    self.t = nil
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def self.show sid, au = nil
    res = Story.find sid
    if au
      raise ActiveRecord::Forbidden unless res.own?(au)
    end
    res
  end
  
  def self.new_t comic_id
    r = Story.max_t(comic_id)
    r.blank? ? 0 : r.to_i + 1
  end
  
  def self.max_t comic_id
    Story.maximum(:t, :conditions => ['comic_id = ?', comic_id])
  end
  
  def self.find_t comic_id, t
    Story.find(:first, :conditions => ['comic_id = ? and t = ?', comic_id, t])
  end
  
  def self.collect_t story
    r = Story.find(:all, :conditions => ['comic_id = ?', story.comic_id], :order => 't')
    r.map {|s| s.t}
  end
  
  def self.serial? ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_t story
    Story.serial?(Story.collect_t(story))
  end
  
  def insert_shift
    Story.update_all('t = t + 1', ['comic_id = ? and t >= ?', self.comic_id, self.t])
  end
  
  def lesser_shift old_t
    self.t = 0 if self.t < 0
    Story.update_all('t = t + 1', ['comic_id = ? and (t >= ? and t < ?)', self.comic_id, self.t, old_t])
  end
  
  def higher_shift old_t
    nf = Story.find_t(self.comic_id, self.t)
    max_t = Story.max_t(self.comic_id).to_i
    self.t = max_t if self.t > max_t
    Story.update_all('t = t - 1', ['comic_id = ? and (t > ? and t <= ?)', self.comic_id, old_t, self.t])
  end
  
  def update_shift old_t
    if self.t > old_t
      higher_shift old_t
    else
      lesser_shift old_t
    end
  end
  
  def rotate old_t = nil
    if self.new_record?
      if self.t.blank?
        self.t = Story.new_t self.comic_id
      else
        self.insert_shift
      end
    else
      if self.t.blank?
      else
        self.update_shift old_t
      end
    end
  end
  
  def allow?
    c = self.comic
    pl = self.panel
    c and c.own?(self.author) and pl and pl.usable?(self.author)
  end
  
  def store old_t = nil
    res = false
    raise ActiveRecord::Forbidden unless self.allow?
    Story.transaction do
      self.rotate old_t
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = Story.validate_t(self) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
  def destroy_and_shorten
    Story.transaction do
      Story.update_all('t = t - 1', ['comic_id = ? and (t > ?)', self.comic_id, self.t])
      raise ActiveRecord::Rollback unless self.destroy
    end
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.panel_count cnt, prm = self.default_panel_size
    count = prm.to_i
    count = self.max_panel_size if count > self.max_panel_size
    count = self.default_panel_size if count < 1
    count
  end
  
  def self.list comic, author, offset = 0, limit = Story.default_panel_size
    opt = self.list_opt
    opt.merge!({:conditions => ['stories.comic_id = ?', comic.id], :order => 'stories.t', :offset => offset, :limit => limit})
    Story.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {
      :author => {}, 
      :comic => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:resource_picture => {:artist => {}}}, 
        :speech_balloons =>{:balloons => {}, :speeches => {}}
      }
    }}
  end
  
  def self.list_json_opt
    {:include => {
      :author => {}, 
      :comic => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:resource_picture => {:artist => {}}}, 
        :speech_balloons =>{:balloons => {}, :speeches => {}}
      }
    }}
  end
  
  def to_json_list
    self.to_json( :include => {:author => {}, :panels => {:methods => :panel_element}})
  end
  
  
end
